﻿using System;
using Lucene.Linq.Mapping;
using Lucene.Linq.Storage;
using Lucene.Net.Analysis;
using Lucene.Net.Analysis.Standard;
using Microsoft.WindowsAzure.StorageClient;

namespace Lucene.Linq.Azure.Test.WorkerRole
{
    [Serializable, Document(DefaultAnalyzer = typeof(StandardAnalyzer))]
    public class Job : TableServiceEntity, IEntity
    {
        [Field(FieldIndex.UnTokenized, FieldStore.Yes, Analyzer = typeof(SimpleAnalyzer), IsDefault = false)]
        public string Id { get; set; }

        [Field(FieldIndex.Tokenized, FieldStore.Yes, Analyzer = typeof(SimpleAnalyzer), IsDefault = false)]
        public string Name { get; set; }

        /// <summary>
        /// Default constructor, we need this when we serialize and especially deserialize objects
        /// </summary>
        public Job()
        { }

        /// <summary>
        /// Specialized constructor, so I can show the example with very few lines of code
        /// </summary>
        /// <param name="id">id of person</param>
        /// <param name="name">name of person</param>
        public Job(string id, string name)
        {
            Id = id;
            Name = name;
        }

        /// <summary>
        /// Content type for entity
        /// </summary>
        public string ContentType
        {
            get { return Storage.ContentType.Binary; }
            set
            {
                // nothing to do
            }
        }

        /// <summary>
        /// Key of entity
        /// </summary>
        [Field(FieldIndex.UnTokenized, FieldStore.Yes, IsKey = true, IsDefault = true, Analyzer = typeof(SimpleAnalyzer))]
        public string Key
        {
            get { return MakeKey(Id); }
            set
            {
                // do nothing... But this is required because of deserialization
            }
        }

        /// <summary>
        /// Just make some string that we can use as a key
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public static string MakeKey(string Id)
        {
            return string.Format("job{0}", Id);
        }
    }
}
