﻿using System;
using System.Linq.Expressions;
using Lucene.Net.Analysis;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Index;
using Lucene.Net.Search;

namespace Lucene.Linq.Expressions
{
    /// <summary>
    /// TODO:Implement properly
    /// </summary>
    internal class ObjectQueryTranslator : LuceneExpressionVisitor, IQueryTranslator
    {
        private readonly BooleanQuery _root;
        private Analyzer _defaultAnalyzer;
        private string[] _defaultFieldNames;

        #region Constructors

        public ObjectQueryTranslator(string[] defaultFieldNames, Analyzer defaultAnalyzer)
        {
            _defaultAnalyzer = defaultAnalyzer;
            _defaultFieldNames = defaultFieldNames;

            _root = new BooleanQuery();
        }

        public ObjectQueryTranslator() : this(new string[0], new StandardAnalyzer())
        {
        }

        #endregion

        #region Translate

        public ISearchQuery Translate(Expression expression)
        {
            // visit the expression first to generate the query
            Visit(expression);

            // inspect the root for clause length
            int length = _root.GetClauses().Length;

            switch (length)
            {
                case 0:
                    new RangeQuery(new Term("", "0"), null, true);
                    break;
                case 1:
                    _root.GetClauses()[0].GetQuery();
                    break;
                default:
                    break;
            }

            return null;
        }

        #endregion

        #region Private Methods

        private static Expression StripQuotes(Expression e)
        {
            while (e.NodeType == ExpressionType.Quote)
            {
                e = ((UnaryExpression) e).Operand;
            }

            return e;
        }

        private void VisitStartsWith(MethodCallExpression m)
        {
            string phrase;

            if (m.Object != null && m.Object.Type == typeof (string))
            {
                var field = (FieldExpression) m.Object;
                var constant = (ConstantExpression) m.Arguments[0];
                phrase = (string) constant.Value;

                // TODO: get field analyzer
                string[] tokens = phrase.Tokenize(field.Name, new SimpleAnalyzer());


                int indexLength = phrase.Split(' ').Length > 1 ? 2 : 0;

                AddMust(
                    null
                    );
                //QueryString.Insert(QueryString.Length - phrase.Length - indexLength, field.Name + ":(");
                //QueryString.Append("*)");
            }
            else
            {
                var constant = (ConstantExpression) m.Arguments[1];
                phrase = (string) constant.Value;

                //QueryString.Append("(");
                Visit(m.Arguments[1]);
                //QueryString.Append("*)");
            }

            // For multi-phrase queries, remove the quotation marks that would break it
            if (phrase.Split(' ').Length > 1)
            {
                //QueryString.Remove(QueryString.ToString().LastIndexOf(phrase) - 1, 1);              // Left Quotation Mark
                //QueryString.Remove(QueryString.ToString().LastIndexOf(phrase) + phrase.Length, 1);  // Right Quation Mark
            }
        }

        #region Add Query

        private void AddMust(Query query)
        {
            _root.Add(query, BooleanClause.Occur.MUST);
        }

        private void AddShould(Query query)
        {
            _root.Add(query, BooleanClause.Occur.SHOULD);
        }

        private void AddMustNot(Query query)
        {
            _root.Add(query, BooleanClause.Occur.MUST_NOT);
        }

        #endregion

        #endregion

        protected override Expression VisitMethodCall(MethodCallExpression m)
        {
            if (!(m.Arguments[0] is MemberInitExpression))
                Visit(m.Arguments[0]);

            //if (QueryString.ToString().EndsWith(")"))
            //    QueryString.Append(" ");

            switch (m.Method.Name)
            {
                case "StartsWith":
                    VisitStartsWith(m);
                    break;
                case "Like":
                    Visit(m.Arguments[1]);
                    //QueryString.Append("~");

                    //if (m.Arguments.Count == 3)
                    //    QueryString.Append(m.Arguments[2]);

                    break;
                case "Match":
                    Visit(m.Arguments[1]);
                    break;
                case "Between":
                    //QueryString.Append("{");
                    //this.Visit(m.Arguments[1]);
                    //QueryString.Append(" TO ");
                    //this.Visit(m.Arguments[2]);
                    //QueryString.Append("}");
                    break;
                case "Include":
                    //QueryString.Append("[");
                    //this.Visit(m.Arguments[1]);
                    //QueryString.Append(" TO ");
                    //this.Visit(m.Arguments[2]);
                    //QueryString.Append("]");
                    break;
                case "Boost":
                    //QueryString.Append("^");
                    //QueryString.Append(m.Arguments[1]);
                    break;
                case "Search":
                    var constant = (ConstantExpression) m.Arguments[1];
                    var phrase = (string) constant.Value;

                    //QueryString.Append(m.Arguments[1]);
                    //QueryString.Remove(QueryString.ToString().LastIndexOf(phrase) - 1, 1);              // Left Quotation Mark
                    //QueryString.Remove(QueryString.ToString().LastIndexOf(phrase) + phrase.Length, 1);  // Right Quation Mark
                    break;
                default:
                    throw new NotSupportedException(string.Format(
                        "Method '{0}' has no supported translation to Lucene", m.Method.Name));
            }

            return m;
        }

        protected override Expression VisitUnary(UnaryExpression u)
        {
            switch (u.NodeType)
            {
                case ExpressionType.Not:
                    //if (!QueryString.ToString().EndsWith("(") && !QueryString.ToString().EndsWith(" "))
                    //    QueryString.Append(" ");

                    //QueryString.Append("-");
                    Visit(u.Operand);
                    break;
                default:
                    throw new NotSupportedException(string.Format("The unary operator '{0}' is not supported",
                                                                  u.NodeType));
            }

            return u;
        }

        protected override Expression VisitBinary(BinaryExpression b)
        {
            //QueryString.Append("(");
            Visit(b.Left);
            //switch (b.NodeType) {
            //    case ExpressionType.And:
            //    case ExpressionType.AndAlso:
            //        QueryString.Append(" AND ");
            //        break;
            //    case ExpressionType.Or:
            //    case ExpressionType.OrElse:
            //        QueryString.Append(" OR ");
            //        break;
            //    case ExpressionType.Equal:
            //        break;
            //    case ExpressionType.Not:
            //        QueryString.Append("NOT ");
            //        break;
            //    case ExpressionType.NotEqual:
            //        int index = QueryString.ToString().LastIndexOf(" ");
            //        QueryString.Insert(index + 1, "-");
            //        break;
            //    default:
            //        throw new NotSupportedException(string.Format("The binary operator '{0}' is not supported", b.NodeType));
            //}
            Visit(b.Right);
            //QueryString.Append(")");
            return b;
        }

        protected override Expression VisitConstant(ConstantExpression c)
        {
            //switch (Type.GetTypeCode(c.Value.GetType())) {
            //    case TypeCode.String:
            //        string phrase = c.Value as string;
            //        if (phrase.Split(' ').Length > 1)
            //            QueryString.Append("\"").Append(c.Value).Append("\"");
            //        else
            //            QueryString.Append(c.Value);
            //        break;
            //    case TypeCode.Object:
            //        string[] args = c.Value as string[];

            //        if (args != null) {
            //            QueryString.Append("(");
            //            foreach (string s in args) {
            //                if (s.Split(' ').Length > 1) {
            //                    QueryString.Append("\"");

            //                    int index = s.LastIndexOf("^");
            //                    if (index != -1) {
            //                        string m = s.Insert(index, "\"");
            //                        QueryString.Append(m);
            //                    } else {
            //                        QueryString.Append(s).Append("\"");
            //                    }
            //                } else {
            //                    QueryString.Append(s);
            //                }

            //                QueryString.Append(" ");
            //            }

            //            QueryString.Remove(QueryString.Length - 1, 1);
            //            QueryString.Append(")");
            //        }
            //        break;
            //    case TypeCode.Boolean:
            //        QueryString.Append("(");
            //        QueryString.Append("\"");
            //        QueryString.Append(((bool)c.Value) ? "1" : "0");
            //        QueryString.Append("\"");
            //        QueryString.Append(")");
            //        break;
            //    default:
            //        throw new NotSupportedException(string.Format("The constant for '{0}' is not supported", c.Value));
            //        break;
            //}

            return c;
        }

        protected override Expression VisitField(FieldExpression field)
        {
            //QueryString.Append(field.Name).Append(":");
            return field;
        }

        protected override Expression VisitSelect(SelectExpression select)
        {
            if (select.DefaultFieldNames != null)
            {
                _defaultFieldNames = select.DefaultFieldNames;
            }

            if (select.From != null)
            {
                VisitSource(select.From);
            }

            if (select.Where != null)
            {
                Visit(select.Where);
            }

            return select;
        }

        protected override Expression VisitSource(Expression source)
        {
            switch ((LuceneExpressionType) source.NodeType)
            {
                case LuceneExpressionType.Index:
                    var index = (IndexExpression) source;
                    //sb.Append(index.Name);
                    //sb.Append(" AS ");
                    //sb.Append(index.Alias);
                    break;
                case LuceneExpressionType.Select:
                    var select = (SelectExpression) source;
                    //sb.Append("(");
                    Visit(select);
                    //sb.Append(")");
                    //sb.Append(" AS ");
                    //sb.Append(select.Alias);
                    break;
                default:
                    throw new InvalidOperationException("Select source is not valid type");
            }
            return source;
        }
    }
}