﻿using System;
using System.Collections.Generic;

namespace Lucene.Linq.Search
{
    /// <summary>
    /// A set of indexes
    /// </summary>
    public interface IIndexSet
    {
        /// <summary>The number of indexes in the set</summary>
        int Count { get; }

        /// <summary>
        /// Gets the IIndex based on the type
        /// </summary>
        /// <param name="t">the key type</param>
        /// <returns>found index reference, otherwise null</returns>
        IIndex this[Type t] { get; }

        /// <summary>Add new Index</summary>
        /// <param name="indexType">type of document the index stores</param>
        void Add(Type indexType);

        /// <summary>Add new Index</summary>
        /// <typeparam name="TIndex">type of document the index stores</typeparam>
        void Add<TIndex>();

        /// <summary>Returns all the Indexes in the set</summary>
        IEnumerable<IIndex> All();

        /// <summary>
        /// Gets the index of a particular type from the set. Exception is thrown if the TIndex isn't stored in the type
        /// </summary>
        /// <param name="t">Type of the index</param>
        /// <returns>Instance of the index</returns>
        IIndex Get(Type t);

        /// <summary>
        /// Gets the index of a particular type from the set. Exception is thrown if the TIndex isn't stored in the type
        /// </summary>
        /// <typeparam name="TIndex">Type of the index</typeparam>
        /// <returns>Instance of the index</returns>
        IIndex<TIndex> Get<TIndex>();

        /// <summary>Removes an index from the set</summary>
        /// <param name="indexType">the type of the index</param>
        void Remove(Type indexType);

        /// <summary>Removes an index from the set</summary>
        /// <typeparam name="TIndex">the type of the index</typeparam>
        void Remove<TIndex>();

        /// <summary>
        /// Checks if an index of a particular type is known
        /// </summary>
        /// <typeparam name="TIndex">Type of the index</typeparam>
        /// <returns>True if the type is known</returns>
        bool Exists<TIndex>();
    }
}